<?php
namespace Teknisa\COZ\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Teknisa\Model\Geral\Entities\Lista;

class Pla10510Service {

    protected $util;
    protected $dataSourceManager;
    protected $environment;
    protected $entityManager;
    protected $novoCodigo;
    protected $pla10510remoteRepository;
    
    public function __construct($util, $dataSourceManager, $environment, $entityManager, $novoCodigo, $pla10510remoteRepository) {
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        $this->pla10510remoteRepository = $pla10510remoteRepository;
    }

    public function geraListaServico($row) {

        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $row['NRORG']);

        $servicosFiliais = $row['SERVICOFILIAL'];

        $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";

        foreach ($servicosFiliais as $servicoFilial) {

            $stmt = $this->entityManager->getConnection()->prepare($sql);
            $stmt->execute(
                array(
                    "NRLISTA" => $nrlista,
                    "CDFILIAL" => $servicoFilial['CDFILIAL'],
                    "NRORG" => $row['NRORG'],
                    "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
                    "NMSERVICO" => $servicoFilial['NMSERVICO'],
                    "CDGRUPSERV" => $servicoFilial['CDGRUPSERV']
                )
            );

        }
        return $nrlista;

    }

    public function geraLista($nrorg, $lista, $separador) {
        if ($lista) {
            $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
            $lista = explode($separador, $lista);
            $this->entityManager->getConnection()->beginTransaction();
            foreach ($lista as $item) {
                $lista = new Lista();
                $lista->setNrlista($nrlista);
                $lista->setCdlista($item);

                $lista->setNrorg($nrorg);
                $lista->setNrorginclusao($nrorg);
                $lista->setNrorgultatu($nrorg);
                $lista->setDtinclusao(new \DateTime());
                $lista->setDtultatu(new \DateTime());
                $lista->setCdoperinclusao($this->environment->getCdOperador());
                $lista->setCdoperultatu($this->environment->getCdOperador());
                $lista->setIdativo('S');

                $this->entityManager->persist($lista);
                $this->entityManager->flush();
            }

            $this->entityManager->getConnection()->commit();

            return $nrlista;
        }
    }

    public function deletaListaServico($nrorg, $nrlista) {

        $sql = "DELETE FROM LISTA_SERVICO WHERE NRLISTA = :NRLISTA AND NRORG = :NRORG";
        $stmt = $this->entityManager->getConnection()->prepare($sql);
        $stmt->execute(
            array(
                "NRLISTA" => $nrlista,
                "NRORG" => $nrorg
            )
        );

    }

    public function deletaLista($nrorg, $nrlista) {
        $lista = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Lista')
            ->findBy(array('nrorg' => $nrorg, 'nrlista' => $nrlista));

        $this->entityManager->getConnection()->beginTransaction();

        foreach ($lista as $registro) {
            $this->entityManager->remove($registro);
            $this->entityManager->flush();
        }

        $this->entityManager->getConnection()->commit();

        return true;
    }

    public function preencheCampos(&$row){
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        if ($row['CDGRUPFILI'] === '') {
            $row['CDGRUPFILI'] = 'T';
        }

        $row['LISTAGRUPFILI'] = $row['CDGRUPFILI'];
        $row['LISTAFILIAGREGA'] = $this->retornaFiliaisagregadas((array)$row);
        $row['LISTAGRUPSERV'] = $this->retornaGrupserv((array)$row);
        $row['SERVICOFILIAL'] = $this->retornaServicoporfiliais((array)$row);
        $row['LISTASERVFILI'] = $this->concatArray($row['SERVICOFILIAL'], 'SERVICO');
        $row['CDTIPOPRAT'] = count($row['NMTIPOPRAT']) == 0 ? 'T': $this->concatArray($row['NMTIPOPRAT']);
    }

    public function retornaFiliaisagregadas($row){
        if(count($row['NMFILIAGREGA']) == 0){   
            $row['CDGRUPFILI'] = $row['LISTAGRUPFILI'];

            //valores que o filtro de filiais agregadar precisa
            $fieldsToFilter = array("NRORG", "CDGRUPFILI", "CDCENTRAL");

            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);

            $row['NMFILIAGREGA'] = $this->pla10510remoteRepository->fetchFiliaisagregadas($paramObjectArray);
        }
        return $this->concatArray($row['NMFILIAGREGA']);
    }

    public function retornaGrupserv($row){
        if(count($row['NMGRUPSERV']) == 0){   
            $row['NMGRUPSERV'] = $this->pla10510remoteRepository->fetchGrupserv();
        }
        return $this->concatArray($row['NMGRUPSERV']);
    }

    public function retornaServicoporfiliais($row){
        if(count($row['NMSERVFILI']) == 0){
            $row['CDGRUPFILI'] = $row['LISTAGRUPFILI'];
            $row['CDGRUPSERV'] = $row['LISTAGRUPSERV'];
            $row['CDFILIAGREGA'] = $row['LISTAFILIAGREGA'];

            //valores que o filtro de serviços precisa
            $fieldsToFilter = array("NRORG", "CDGRUPFILI", "CDCENTRAL", "CDGRUPSERV", "CDFILIAGREGA");

            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);

            $row['SERVICOFILIAL'] = $this->pla10510remoteRepository->fetchServicos($paramObjectArray);
        }
        return $row['SERVICOFILIAL'];
    }

    public function concatArray($array, $key = false){
        if ($key){
            $string = '';
            foreach ($array as $item) {
              $string .= $item[$key].'_';
            }
            return trim($string, '_');
        }
        else
        {
            return implode('_', $array);
        }
    }

    /*
     * Constroi um array de arrays a partir de uma row, no formato chave=>valor,
     * e um array de strings especificando quais das colunas deverão ser usadas
     */
    public function buildFilterArrayFromRow($row, $filterKeys){
 
        //pega todos os valores de $row com chaves presentes em $filterKeys
        //o array_flip faz com que os valores de $filterKeys sejam chaves
        $paramValueArray = array_intersect_key($row, array_flip($filterKeys));

        return array_map(function($key, $value){
            return array("columnName"=>$key, "operator"=>"=", "value"=>$value); 
        }, array_keys($paramValueArray), $paramValueArray);
    }
}